! ############################################################
!            Satellite Data Simulation Unit v2
!                     -  convolv_visir -
!
! This subroutine applies beam convolution to visible/IR radiance.
! The output file will have the same name as the original 
! except that the extention ".bin" is replaced by ".fov.bin". 
! Note that spatial sampling (or grid dimension and interval) 
! remains unchanged.
!
!              Updated from convolv-fov.f by H. M. (May, 2009)
!
 Subroutine convolv_visir
   Use MAINparameters
   Use BMCVparameters
   Implicit NONE
!
! Local variables
!
   Real :: radout
   Real :: FWHMx, FWHMy
   Integer :: nx, ny, iw
!
! Functions
! 
   Real :: wl,Ib,T
   Real :: Tbb, plnk
!
   Tbb(wl,Ib)=1.438786e4/wl/log(1.1911e8/wl**5/Ib+1.)
   plnk(wl,T)=1.1911e8/wl**5/(exp(1.438786e4/T/wl)-1.)
   allocate ( radinp(ngridx,ngridy) )
!
   print *, ' --- Visible/IR FOV convolution --- '
!
   Do iw = 1 , mxwavel
!
      FWHMx = fov_ct_visir(iw)
      FWHMy = fov_dt_visir(iw)
!
      If ( tbform(iw) ) then 
!
         Do nx = 1 , ngridx
            Do ny = 1 , ngridy
               radinp(nx,ny) = plnk(wavel(iw),Ivir_out(nx,ny,iw))
            End do
         End do
!
         Do nx = 1 , ngridx
            Do ny = 1 , ngridy
!
               Call fov(nx,ny,FWHMx,FWHMy,radout)
               Ivir_out(nx,ny,iw) = Tbb(wavel(iw),radout)
!
            End do
         End do
!
      Else
!
         radinp = Ivir_out(:,:,iw)
         Do nx = 1 , ngridx
            Do ny = 1 , ngridy
!
               Call fov(nx,ny,FWHMx,FWHMy,radout)
               Ivir_out(nx,ny,iw) = radout
!
            End do
         End do
!
      End If
!
   End do
!
   deallocate ( radinp )
!
   Return
 End Subroutine convolv_visir

