! ############################################################
!            Satellite Data Simulation Unit v2
!                         - fov -
!
! This subroutine applies a given beam pattern to synthesized 
! measurement. The beam pattern is defined by 'beamptrn' in
! the end of this file.
!
!              Updated from convolv-fov.f by H. M. (May, 2009)
!
 Subroutine fov(nx,ny,FWHMx,FWHMy,radout)
   Use MAINparameters
   Use BMCVparameters
   Implicit none
!
! I/O parameters
!
   Integer, intent(in) :: nx,ny
   Real, intent(in) :: FWHMx,FWHMy
   Real, intent(out) :: radout
!
! Local variables
!
   Integer :: nxsub,nysub,nxbegin,nxend,nybegin,nyend,i,j
   Real :: normlz,dx,dy
!
   nxsub = nint(FWHMx / deltax)
   nysub = nint(FWHMy / deltay)
   nxbegin = max(1,nx-nxsub)
   nybegin = max(1,ny-nysub)
   nxend   = min(ngridx,nx+nxsub)
   nyend   = min(ngridy,ny+nysub)
   radout  = 0. ;  normlz = 0.
!
   Do i = nxbegin, nxend
      Do j = nybegin, nyend
         dx = (nx-i) * deltax
         dy = (ny-j) * deltay
         radout = radout + deltax * deltay * beamptrn(dx,dy) * radinp(i,j)
         normlz = normlz + deltax * deltay * beamptrn(dx,dy)
      End do
   End do
   radout = radout / normlz
!
   Contains
!
! - Beam (antenna) pattern
!
     Real Function beamptrn (x,y)
       Implicit NONE
!
       Real :: x,y
!
! Gaussian beam pattern
!
      beamptrn = exp( -((x/FWHMx)**2 + (y/FWHMy)**2)*4*log(2.) )
!
      Return
    End Function beamptrn
!
 End Subroutine fov
