! ############################################################
!            Satellite Data Simulation Unit
!                     -  SDSU_V2 -
!
!  This program is the driver of the whole simulator unit.
!
!                   Updated from RTSIM.f by H.M. (May, 2009)
! ------------------------------------------------------------
!
 Program SDSU_V2
!
! - Common variables are declared and their dimensions are defined.
!
   Use MAINparameters
   Implicit none
!
   integer :: ifile
!
! - Sensor parameters and running options
!
   Call param_set
!
! - Microphyscial parameters
!
   Call microp_set
!
! - FOV parameters
!
   If ( beamconv ) &
   Call param_set_bmcv
!
! - Input file list
!
   Call def_file_list
!
! - Simulation begins ------------------------------------------
!
   Do ifile = 1, nmodel
!
! - Input geophysical parameters
!
      Call rd_CRM ( ifile )
!
! ##############################################################
!      --- MICROWAVE BRIGHTNESS TEMPERATURE SIMULATION ---    
!        Subroutines called here are found in './microw/'
!        and in '../beamconv/'
! ##############################################################
!
      If ( microw ) then
!
! - Microwave radiative transfer solver
!
         Call SIMmicro
!
! - Output
!
         Call out_micro ( ifile )
!
      Endif
!
      If ( microw .and. beamconv ) then
!
! - Beam convolution
!
         Call convolv_micro
!
! - Output
!
         Call out_micro_bmcv ( ifile )
!
      Endif
!
! ##############################################################
!                --- RADAR ECHO SIMULATION ---
!        Subroutines called here are found in './radar/'
!        and in '../beamconv/'
! ##############################################################
!
      If ( radar ) then
!
! - Radar echo simulator
!
         Call SIMradar
!
! - Output
!
         Call out_radar ( ifile )
!
      Endif
!
      If ( radar .and. beamconv ) then
!
! - Beam convolution
!
         Call convolv_radar
!
! - Output
!
         Call out_radar_bmcv ( ifile )
!
      Endif
!
! ##############################################################
!       --- VISIBLE AND INFRARED RADIANCE SIMULATION ---
!        Subroutines called here are found in './vis_IR/'
!        and in '../beamconv/'
! ##############################################################
!
      If ( visIR ) then
!
! - Visible/IR radative transfer solver
!
         Call SIMvisir
!
! - Output
!
         Call out_visIR ( ifile )
!
      Endif
!
      If ( visIR .and. beamconv ) then
!
! - Beam convolution
!
         Call convolv_visir
!
! - Output
!
         Call out_visir_bmcv ( ifile )
!
      Endif
!
! --------------------------------------------------------------
!
! - Memory deallocation for input CRM parameters
!
      Call dealloc_CRM
!
    End do
!
  End Program SDSU_V2
