! ############################################################
!            Satellite Data Simulation Unit v2
!                     -  out_visIR -
!
! This subroutine writes out the computed visble/IR radiance
!
!                 Updated from out-visIR.f by H. M. (May, 2009)
!
 Subroutine out_visIR ( ifile )
   Use MAINparameters
   Use MCRPparameters
   Implicit NONE
!
! I/O parameters
!
   Integer, intent(in) :: ifile
!
! Local variables
!
   Integer :: i,j,nw
   character*90 :: pathandfile, filename
!
   filename = CRM_file_list(ifile)
!
! Data dimensions in ascii format
!
   pathandfile = outputdir//trim(filename)//'.vir.info'
   open(unit= 17, file = pathandfile, form = 'formatted')
!
   write(17,51) ngridx, ngridy, nlyr, deltax, deltay
   write(17,*) 'solar zenith angle:',znth_slr
   write(17,*) 'viewing zenith angle:',znth_obs
   write(17,*) 'viewing-to-solar azimuth angle:',azmth
   write(17,*) mxwavel,' wavelengths at', wavel,' micron'
!
   write(17,*) '/ Microphysical setup /'
   write(17,*) ' - Cloud ice: ',trim(cldi%name)//'('//cldi%functype//')'
   Do i = 1, cldi%nparams
      write(17,52) '  PSD parameter #',i,'=',cldi%paramset(i)
   End do
   write(17,*) ' - Cloud water: ',trim(cldw%name)//'('//cldw%functype//')'
   Do i = 1, cldw%nparams
      write(17,52) '  PSD parameter #',i,'=',cldw%paramset(i)
   End do
   write(17,*) ' - Rain: ',trim(rain%name)//'('//rain%functype//')'
   Do i = 1, rain%nparams
      write(17,52) '  PSD parameter #',i,'=',rain%paramset(i)
   End do
   write(17,*) ' - Snow: ',trim(snow%name)//'('//snow%functype//')'
   Do i = 1, snow%nparams
      write(17,52) '  PSD parameter #',i,'=',snow%paramset(i)
   End do
   write(17,*) ' - Graupel: ',trim(grpl%name)//'('//grpl%functype//')'
   Do i = 1, grpl%nparams
      write(17,52) '  PSD parameter #',i,'=',grpl%paramset(i)
   End do
   write(17,*) ' - Hail: ',trim(hail%name)//'('//hail%functype//')'
   Do i = 1, hail%nparams
      write(17,52) '  PSD parameter #',i,'=',hail%paramset(i)
   End do
!
! Computed radiances in binary format
!
   pathandfile = outputdir//trim(filename)//'.vir.org'
   open(unit= 22, file = pathandfile, &
        access='direct', recl=ngridx*ngridy*mxwavel*4)
!
   write(22,rec=1) (((Ivir_out(i,j,nw),i=1,ngridx), &
                         j=1,ngridy),nw=1,mxwavel)
!
   Close (17)
   Close (22)
!
51 format(i3,' x ',i3,' (x,y) grid; ',i3,' layers; ', &
          ' deltax = ',f6.2,' km','  deltay = ',f6.2,' km')
52 format(a,i2,a,1P1e10.3)
   If ( .not. beamconv ) deallocate ( Ivir_out )
!
   Return
 End Subroutine out_visIR

